import * as PIXI from 'pixi.js';
import * as p2 from 'p2'
import './style.css'

function randomInt(max) {
  return Math.floor(Math.random() * (max + 1))
}

const vw = Math.max(document.documentElement.clientWidth || 0, window.innerWidth || 0)
const vh = Math.max(document.documentElement.clientHeight || 0, window.innerHeight || 0)

let app = new PIXI.Application({
  width: vw,
  height: vh,
  antialias: true,
  backgroundColor: 0xffffff
});
document.body.appendChild(app.view);


class PhysicsObject {
  body;
  shape;
  graphics;
  parentContainer;

  constructor(body, shape, graphics) {
    this.body = body;
    this.shape = shape;
    this.graphics = graphics;

    this.body.addShape(this.shape);

    this.graphics.interactive = true;
    this.graphics.on('click', (e) => {
      this.body.applyImpulse([0, 25]);
    })
  }

  assign(world, container) {
    world.addBody(this.body);
    container.addChild(this.graphics);
  }

  update() {
    // Transfer positions of the physics objects to Pixi.js
    this.graphics.position.x = this.body.position[0];
    this.graphics.position.y = this.body.position[1];
    this.graphics.rotation = this.body.angle;
  }
}

let worldObjects = [];

let zoom = 15;
let world = new p2.World({ gravity: [0, -9.81] });
// Add a plane

// let planeShape = new p2.Plane();
// let planeBody = new p2.Body({
//   position: [0, -2]
// });
// planeBody.addShape(planeShape);
// world.addBody(planeBody);

let container = new PIXI.Container();
container.interactive = true;
container.position.x = app.renderer.width / 2; // center at origin
container.position.y = app.renderer.height / 2 + 50; 
container.scale.x = zoom; // zoom in
container.scale.y = -zoom; // Note: we flip the y axis to make "up" the physics "up"
container.on('mousedown', e => {
  console.log(e.data.global.x, e.data.global.y)
})
app.stage.addChild(container);

// Add a box
let box = new PhysicsObject(
  new p2.Body({
    mass: 1,
    position: [0, 2],
    angularVelocity: 1
  }),
  new p2.Box({
    width: 2,
    height: 1
  }),
  new PIXI.Graphics()
)
box.graphics.beginFill(randomInt(0xffffff));
box.graphics.drawRect(
  -box.shape.width / 2, -box.shape.height / 2,
  box.shape.width, box.shape.height);
box.graphics.endFill();

box.assign(world, container);
worldObjects.push(box);

let box2 = new PhysicsObject(
  new p2.Body({
    mass: 2,
    position: [-1, 3],
    angularVelocity: 0
  }),
  new p2.Box({
    width: 2,
    height: 1
  }),
  new PIXI.Graphics()
)
box2.graphics.beginFill(0x00ff00);
box2.graphics.drawRect(
  -box.shape.width / 2, -box.shape.height / 2,
  box.shape.width, box.shape.height);
box2.graphics.endFill();

box2.assign(world, container);
worldObjects.push(box2);


// static ground box
let box3 = new PhysicsObject(
  new p2.Body({
    mass: 0,
    position: [0, -2]
  }),
  new p2.Box({
    width: 50,
    height: 0.2
  }),
  new PIXI.Graphics()
)
box3.graphics.beginFill(0x666666);
box3.graphics.drawRect(
  -box3.shape.width / 2, -box3.shape.height / 2,
  box3.shape.width, box3.shape.height);
box3.graphics.endFill();

box3.assign(world, container);
worldObjects.push(box3);

app.ticker.add((delta) => {
  world.step(1 / 60);
  worldObjects.forEach(o => o.update());
});

function randomBox() {
  let width = 1 + Math.random() * 3;
  let height = 1 + Math.random() * 3;
  // Add a box
  let box = new PhysicsObject(
    new p2.Body({
      mass: 0.5 * Math.sqrt(width * height),
      position: [randomInt(25) - 12.5, randomInt(10) + 5],
      angularVelocity: 0
    }),
    new p2.Box({
      width: width,
      height: height
    }),
    new PIXI.Graphics()
  )
  box.graphics.beginFill(randomInt(0xffffff));
  box.graphics.drawRect(
    -box.shape.width / 2, -box.shape.height / 2,
    box.shape.width, box.shape.height);
  box.graphics.endFill();

  box.assign(world, container);
  worldObjects.push(box);

  return box;
}

function doZoom(e) {
  zoom += -e.deltaY * 0.05;
  zoom = Math.min(Math.max(1, zoom), 100)
  container.scale.x = zoom; // zoom in
  container.scale.y = -zoom; // Note: we flip the y axis to make "up" the physics 
}
window.addEventListener('wheel', doZoom)
window.addEventListener('click', randomBox)